'use strict';

var getReplacers = require('./replace');

var _require = require('./helpers'),
    isCodegenComment = _require.isCodegenComment,
    looksLike = _require.looksLike;

module.exports = codegenPlugin;

function codegenPlugin(babel) {
  var _getReplacers = getReplacers(babel),
      asProgram = _getReplacers.asProgram,
      asIdentifier = _getReplacers.asIdentifier,
      asImportDeclaration = _getReplacers.asImportDeclaration;

  return {
    name: 'codegen',
    visitor: {
      Program(path, _ref) {
        var fileOpts = _ref.file.opts;

        var firstNode = path.node.body[0] || {};
        var comments = firstNode.leadingComments || [];
        var isCodegen = comments.some(isCodegenComment);

        if (isCodegen) {
          comments.find(isCodegenComment).value = ' this file was codegened';
          asProgram(path, fileOpts);
        }
      },
      Identifier(path, _ref2) {
        var fileOpts = _ref2.file.opts;

        var isCodegen = path.node.name === 'codegen';
        if (isCodegen) {
          asIdentifier(path, fileOpts);
        }
      },
      ImportDeclaration(path, _ref3) {
        var fileOpts = _ref3.file.opts;

        var isCodegen = looksLike(path, {
          node: {
            source: {
              leadingComments(comments) {
                return comments && comments.some(isCodegenComment);
              }
            }
          }
        });
        if (isCodegen) {
          asImportDeclaration(path, fileOpts);
        }
      }
    }
  };
}